@@ -1,20 +1,13 @@
-MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold = 0.05 , test = NULL , ini = NULL, user_test = NULL, hash=FALSE, hashObject=NULL, slopes = FALSE, ncores = 1)
+MMPC.temporal = function(target , reps = NULL, group, dataset , max_k = 3 , threshold = 0.05 , test = NULL, ini = NULL, wei = NULL, user_test = NULL, hash=FALSE, hashObject=NULL, slopes = FALSE, ncores = 1)
 {
  #get the log threshold
   threshold = log(threshold)
       
   ##############################
-  # initialization part of SES #
+  # initialization part of MMPC #
   ##############################
 
-  faster = 0;
-  #assign("gRbaseON",0,envir = .GlobalEnv)
-  options(warn=-1)
-  if(requireNamespace("gRbase", quietly = TRUE, warn.conflicts = FALSE) == TRUE)
-  {
-    #assign("gRbaseON",1,envir = .GlobalEnv)
-    faster = 1;
-  }
+
   options(warn=0);
 
   equal_case = 3;
@@ -50,16 +43,17 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
   if(!is.null(dataset))
   {
     #check if dataset is an ExpressionSet object of Biobase package
-    if(class(dataset) == "ExpressionSet")
-    {
-      #get the elements (numeric matrix) of the current ExpressionSet object.
-      dataset = Biobase::exprs(dataset);
-      dataset = t(dataset); #take the features as columns and the samples as rows
-    } else if ( class(dataset) == "matrix" | class(dataset) == "data.frame" ){
-      target = target 
-    } else {
-      stop('Invalid dataset class. It must be either a matrix, a dataframe or an ExpressionSet');
-    }
+    #if(class(dataset) == "ExpressionSet")
+    #{
+    #  #get the elements (numeric matrix) of the current ExpressionSet object.
+    #  dataset = Biobase::exprs(dataset);
+    #  dataset = t(dataset); #take the features as columns and the samples as rows
+    #} else if ( class(dataset) == "matrix" | class(dataset) == "data.frame" ){
+    #  target = target 
+    #} else {
+    #  stop('Invalid dataset class. It must be either a matrix, a dataframe or an ExpressionSet');
+    #}
+	
   }
     if(is.null(dataset) || is.null(target) )
     {
@@ -83,7 +77,7 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
   targetID = -1;
   
   #check if the target is a string
-  if (is.character(target) && length(target) == 1){
+  if (is.character(target) & length(target) == 1){
     findingTarget <- target == colnames(dataset);#findingTarget <- target %in% colnames(dataset);
     if(!sum(findingTarget)==1){
       warning('Target name not in colnames or it appears multiple times');
@@ -94,7 +88,7 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
   }
   
   #checking if target is a single number
-  if (is.numeric(target) && length(target) == 1){
+  if (is.numeric(target) & length(target) == 1){
     if(target > dim(dataset)[2]){
       warning('Target index larger than the number of variables');
       return(NULL);
@@ -283,7 +277,7 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
   }
 
   #call the main MMPC.temporal function after the checks and the initializations
-  results = InternalMMPC.temporal(target, reps, group, dataset, max_k, threshold , test, ini, user_test, dataInfo, hash, varsize, stat_hash, pvalue_hash, targetID, faster, slopes = slopes, ncores = ncores);
+  results = InternalMMPC.temporal(target, reps, group, dataset, max_k, threshold, test, ini, wei, user_test, dataInfo, hash, varsize, stat_hash, pvalue_hash, targetID, slopes = slopes, ncores = ncores);
   
   MMPC.temporal.output <-new("MMPC.temporal.output", selectedVars = results$selectedVars, selectedVarsOrder=results$selectedVarsOrder, hashObject=results$hashObject, pvalues=results$pvalues, stats=results$stats, univ = results$univ, max_k=results$max_k, threshold = results$threshold, runtime=results$runtime, test=ci_test, slope = slopes);
   
@@ -293,7 +287,7 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
 
 #########################################################################################################
 
-  InternalMMPC.temporal = function(target , reps, group, dataset , max_k, threshold , test = NULL , ini, user_test = NULL , dataInfo = NULL , hash=FALSE, varsize, stat_hash, pvalue_hash, targetID, faster, slopes = slopes, ncores = ncores)
+  InternalMMPC.temporal = function(target, reps, group, dataset, max_k, threshold, test = NULL, ini, wei, user_test = NULL, dataInfo = NULL, hash=FALSE, varsize, stat_hash, pvalue_hash, targetID, slopes, ncores)
 {
   #get the current time
   runtime = proc.time();
@@ -306,7 +300,7 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
   #univariate feature selection test
   
   if ( is.null(ini) ) {
-    univariateModels = univariateScore.temporal(target , reps, group, dataset , test, hash=hash, dataInfo, stat_hash=stat_hash, pvalue_hash=pvalue_hash, targetID, slopes = slopes, ncores = ncores);
+    univariateModels = univariateScore.temporal(target , reps, group, dataset , test, wei=wei, hash=hash, dataInfo, stat_hash=stat_hash, pvalue_hash=pvalue_hash, targetID, slopes = slopes, ncores = ncores);
   } else {
     univariateModels = ini
   }
@@ -372,7 +366,7 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
   while(loop)
   {
     #lets find the variable with the max min association
-    max_min_results = max_min_assoc.temporal(target, reps, group, dataset , test , threshold , max_k , selectedVars , pvalues , stats , remainingVars , univariateModels, selectedVarsOrder, hash=hash, dataInfo, stat_hash=stat_hash, pvalue_hash=pvalue_hash, faster, slopes = slopes);
+    max_min_results = max_min_assoc.temporal(target, reps, group, dataset , test , wei, threshold , max_k , selectedVars , pvalues , stats , remainingVars , univariateModels, selectedVarsOrder, hash=hash, dataInfo, stat_hash=stat_hash, pvalue_hash=pvalue_hash, slopes = slopes);
     selectedVar = max_min_results$selected_var;
     selectedPvalue = max_min_results$selected_pvalue;
     remainingVars = max_min_results$remainingVars;
@@ -433,206 +427,8 @@ MMPC.temporal = function(target , reps, group, dataset , max_k = 3 , threshold =
   return(results);
 }
 
-#univariate feature selection ( uncoditional independence )
 
-# univariateScore.temporal = function(target , reps, group, dataset , test, hash, dataInfo, stat_hash=stat_hash, pvalue_hash=pvalue_hash, targetID, slopes, ncores)
-# {
-#   #how many tests
-#   nTests = ncol(dataset);
-#   
-#   #data structure to be returned
-#   univariateModels = NULL;
-#   univariateModels$pvalue = numeric(nTests) 
-#   univariateModels$stat = numeric(nTests)
-#   univariateModels$flag = numeric(nTests) 
-#   #univariateModels$uniModelFit = rep(NA,nTests);
-#   
-#   test_results = NULL;
-#   #for way to initialize the univariateModel
-#   #FOR LOOP IS FASTER THAN VAPPLY IN THIS CASE (try apply withm margin 2)
-#   if ( ncores == 1 | is.null(ncores) | ncores <= 0 ) {
-#     for(i in 1:nTests)
-#     {
-#       #arguments order for any CI test are fixed
-#       if (i != targetID){
-#         test_results = test(target, reps, group, dataset, i, 0, dataInfo = dataInfo, hash = hash, stat_hash = stat_hash, pvalue_hash = pvalue_hash, slopes)
-#         univariateModels$pvalue[[i]] = test_results$pvalue;
-#         univariateModels$stat[[i]] = test_results$stat;
-#         univariateModels$flag[[i]] = test_results$flag;
-#         univariateModels$stat_hash = test_results$stat_hash
-#         univariateModels$pvalue_hash = test_results$pvalue_hash      
-#       }else{
-#         univariateModels$pvalue[[i]] = 1;
-#         univariateModels$stat[[i]] = 0;
-#         univariateModels$flag[[i]] = 1;
-#       }
-#     }
-#     return(univariateModels);
-#   } else {
-#    # require(doParallel, quietly = TRUE, warn.conflicts = FALSE)  
-#     cl <- makePSOCKcluster(ncores)
-#     registerDoParallel(cl)
-#     test = test
-#     mod <- foreach(i = 1:nTests, .combine = rbind) %dopar% {
-#     ## arguments order for any CI test are fixed
-#       if (i != targetID) {
-#         test_results = test(target, reps, group, dataset, i, 0, dataInfo = dataInfo, hash = hash, stat_hash = stat_hash, pvalue_hash = pvalue_hash, slopes)
-#         return( c(test_results$pvalue, test_results$stat, test_results$flag, test_results$stat_hash, test_results$pvalue_hash) )
-#       } else{
-#         return( c(1, 0, 1, test_results$stat_hash, test_results$pvalue_hash) )
-#       }
-#     }
-#     stopCluster(cl)
-#     univariateModels$pvalue = as.vector( mod[, 1] )
-#     univariateModels$stat = as.vector( mod[, 2] )
-#     univariateModels$flag = as.vector( mod[, 3] )
-#     if ( ncol(mod) == 3 ) { 
-#       univariateModels$stat_hash = NULL
-#       univariateModels$pvalue_hash = NULL   
-#     } 
-#   }
-#     return(univariateModels);
-# }
 
-#########################################################################################################
-# 
-# #just like matlab's nchoosek but with transposed result
-# #Its a slightly different from combn() 
-# #(ex. (nchoosekm(4,2) != nchoosekm(1:4,2) like nchoosek in matlab , combn(4,2) == combn(1:4,2)))
-# nchoosekm = function(cs , k, faster) #i can also pass the compFun arg for selecting
-# { 
-#   if(length(cs) == 1) #if not vector
-#   {
-#     res = choose(cs , k); #or nchoosek
-#   }else{
-#     if(faster == 1)
-#     {
-#       res = gRbase::combnPrim(cs,k); #combs(as.vector(cs),k); #combnPrim
-#     }else
-#     {
-#       res = combn(cs,k);
-#     }
-# 
-#   }
-#   return(res);
-# }
-# 
-# #########################################################################################################
-# 
-# compare_p_values = function(pval, pval2, stat, stat2)
-# {
-#   if(length(pval) == 0 | length(pval2) == 0 | length(stat) == 0 | length(stat2) ==0)
-#   {
-#     return(FALSE);
-#   }else{
-#     if(is.na(pval2)==TRUE | is.na(stat2)==TRUE | is.na(pval)==TRUE | is.na(stat)==TRUE)
-#     {
-#       pval2 = 0.0;
-#       return(FALSE);#(pval < pval2);
-#     }else{
-# #       if (pval <= 2e-16 | pval2 <= 2e-16){
-# #         return(stat > stat2);
-# #       }else{
-#         return(pval < pval2);
-#       # }
-#     }
-#   }
-# }
-# 
-# #########################################################################################################
-# 
-# max_min_assoc.temporal = function(target, reps, group, dataset , test , threshold , max_k , selectedVars , pvalues , stats , remainingVars , univariateModels, selectedVarsOrder, hash, dataInfo, stat_hash, pvalue_hash, faster, slopes = slopes)
-# {
-#   #Initialize
-#   selected_var = -1;
-#   selected_pvalue = 2;
-#   selected_stat = 0;
-#   
-#   varsToIterate = which(remainingVars==1);
-#   for(cvar in varsToIterate)
-#   {
-#     mma_res = min_assoc.temporal(target, reps, group, dataset , test , max_k , cvar , selectedVars , pvalues , stats , univariateModels , selectedVarsOrder, hash, dataInfo, stat_hash, pvalue_hash, faster, slopes = slopes);
-#     pvalues = mma_res$pvalues;
-#     stats = mma_res$stats;
-#     stat_hash = mma_res$stat_hash;
-#     pvalue_hash = mma_res$pvalue_hash;
-#     
-#     
-#     if(mma_res$pvalue > threshold)
-#     {
-#       remainingVars[[cvar]] = 0;
-#     }
-#     
-#     if(compare_p_values(mma_res$pvalue , selected_pvalue , mma_res$stat , selected_stat))
-#     {
-#       selected_var = cvar;
-#       selected_pvalue = mma_res$pvalue;
-#       selected_stat = mma_res$stat;
-#     }
-#   }
-#   results <- list(selected_var = selected_var , selected_pvalue = selected_pvalue , remainingVars = remainingVars , pvalues = pvalues , stats = stats, stat_hash=stat_hash, pvalue_hash = pvalue_hash, slope = slopes);
-#   return(results); 
-# }
-# 
-# #########################################################################################################
-# 
-# min_assoc.temporal = function(target , reps, group, dataset , test ,  max_k , cvar , selectedVars , pvalues , stats , univariateModels , selectedVarsOrder, hash, dataInfo, stat_hash, pvalue_hash, faster, slopes = slopes)
-# {
-#   #initialization
-#   #baseline values
-#   #   ma_pvalue = univariateModels$pvalue[[cvar]];
-#   #   ma_stat = univariateModels$stat[[cvar]];
-#   ma_pvalue = pvalues[[cvar]]; #CHANGE
-#   ma_stat = stats[[cvar]]; #CHANGE
-#   
-#   selectedVars = which(selectedVars==1);
-#   #max size of the condiotioning test
-#   k = min(c(max_k , length(selectedVars)));
-#   
-#   ck = 1;
-#   while(ck<=k)
-#   {
-#     #lastvar = unique(which(selectedVarsOrder == max(selectedVarsOrder)));
-#     lastvar = which(selectedVarsOrder == max(selectedVarsOrder))[1]; #CHANGE
-#     
-#     tempCS = setdiff(selectedVars, lastvar) #CHANGE
-#     if(ck == 1) #CHANGE
-#     {
-#       subsetcsk = as.matrix(lastvar); #CHANGE
-#     }else{
-#       subsetcsk = as.matrix(nchoosekm(tempCS,ck-1,faster)); #CHANGE
-#       numSubsets = dim(subsetcsk)[2]; #CHANGE
-#       subsetcsk = rbind(subsetcsk, lastvar*rep(1,numSubsets)); #CHANGE
-#     }
-#     
-#     #or combs or nchoosekm
-#     #subsetcsk = as.matrix(nchoosekm(1:length(selectedVars),ck));
-#     
-#     #subsetcsk = t(subsetcsk);
-#     for(i in 1:ncol(subsetcsk))
-#     {
-#       s = subsetcsk[,i];
-#       s = t(t(s));
-#       
-#       cur_results = test(target , reps, group, dataset , cvar, s , dataInfo=dataInfo, univariateModels, hash = hash, stat_hash, pvalue_hash, slopes = slopes);
-#       stat_hash = cur_results$stat_hash;
-#       pvalue_hash = cur_results$pvalue_hash;
-#       
-#       #check if the pvalues and stats should be updated
-#       if(cur_results$flag == 1 & !compare_p_values(cur_results$pvalue, ma_pvalue, cur_results$stat , ma_stat))
-#       {
-#         ma_pvalue = cur_results$pvalue;
-#         pvalues[[cvar]] = cur_results$pvalue;
-#         
-#         ma_stat = cur_results$stat;
-#         stats[[cvar]] = cur_results$stat;
-#       }
-#     }
-#     ck = ck+1;
-#   }
-#   results <- list(pvalue = ma_pvalue , stat = ma_stat , pvalues = pvalues , stats = stats, stat_hash=stat_hash, pvalue_hash  = pvalue_hash, slope = slopes);
-#   return(results);
-# }
 # 
 # # .onAttach <- function(libname, pkgname){
 # #   # do whatever needs to be done when the package is loaded
