@@ -1,40 +1,44 @@
-bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, ncores = 1 ) {
+bic.fsreg <- function( target, dataset, test = NULL, wei = NULL, tol = 0, robust = FALSE, ncores = 1 ) {
 
   p <- ncol(dataset)  ## number of variables
   bico <- numeric( p )
   moda <- list()
   k <- 1   ## counter
   n <- length(target)  ## sample size
-  con = log(n)
+  con <- log(n)
   tool <- NULL
   info <- matrix( 0, ncol = 2 )
-  result = NULL
-
+  result <- NULL
+  sela <- NULL
+  
   #check for NA values in the dataset and replace them with the variable median or the mode
   if(any(is.na(dataset)) == TRUE)
   {
+
+  
     #dataset = as.matrix(dataset);
     warning("The dataset contains missing values (NA) and they were replaced automatically by the variable (column) median (for numeric) or by the most frequent level (mode) if the variable is factor")
     
-    if(class(dataset) == "matrix")
-    {
-      dataset = apply(dataset, 2, function(x){x[which(is.na(x))] = median(x,na.rm = TRUE)});
+    if (class(dataset) == "matrix")  {
+    
+       dataset = apply(dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x)}) 
+              
     }else{
-      for(i in 1:ncol(dataset))
+	
+    poia <- which( is.na(dataset), arr.ind = TRUE )[2]
+ 	for( i in poia )
       {
-        if(any(is.na(dataset[,i])))
-        {
-          xi = dataset[,i]
+          xi = dataset[, i]
           if(class(xi) == "numeric")
           {                    
-            xi[which(is.na(xi))] = median(xi,na.rm = TRUE) 
-          }else if(class(xi) == "factor"){
-            xi[which(is.na(xi))] = levels(xi)[which.max(xi)]
+            xi[ which( is.na(xi) ) ] = median(xi, na.rm = TRUE) 
+          } else if ( class(xi) == "factor" ) {
+            xi[ which( is.na(xi) ) ] = levels(xi)[ which.max( as.vector( table(xi) ) )]
           }
-          dataset[,i] = xi
+          dataset[, i] = xi
         }
-      }
     }
+    
   }
   
   ##################################
@@ -44,7 +48,7 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
 
   dataset <- as.data.frame(dataset)  ## just in case
   if ( is.null( colnames(dataset) ) )  {  ## checks for column names
-    colnames(x) <- paste("X", 1:p, sep = "")
+    colnames(dataset) <- paste("X", 1:p, sep = "")
   }	
 
   ## dependent (target) variable checking if no test was given, 
@@ -54,119 +58,95 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
 
     ## linear regression 
     if ( class(target) == "numeric" || class(target) == "vector" ) {
-
+     
+	 la <- length( Rfast::sort_unique(target) )
+	 if ( la > 2  &  sum( round(target) - target ) == 0 ) {
+        test <- "testIndPois" 
+		
+	 } else if ( la == 2 ) {
+	    test <- "testIndLogistic"  
+		
       ## percentages
-      if ( min( target ) > 0 & max( target ) < 1 )  {  ## are they percentages?
+     } else if ( min( target ) > 0  &  max( target ) < 1 )  {  ## are they percentages?
         target <- log( target / (1 - target) ) 
-      }
-      test <- "gaussian"  
+     }
+      test <- "testIndReg"  
       
     }
-    
-    ## multivariate data
-    if ( sum( class(target) == "matrix" ) == 1 ) {
-      test <- "gaussian"
-      a <- rowSums(target)
-      if ( min( target ) > 0 & round( sd(a), 16 ) == 0 ) { ## are they compositional data?
-        target <- log( target[, -1] / target[, 1] )
-      }
-    }
-    
+      
     ## surival data
     if ( sum( class(target) == "Surv" ) == 1 ) {
-      test <- "Cox"
+      test <- "censIndCR"
     }
 
-    ## binary data
-    if ( length( unique(target) ) == 2 ) {
-      test <- "binary"   
+    ## ## ordinal, multinomial or perhaps binary data 
+    if ( length( unique(target) ) == 2  &  is.factor(target) ) {
+      test <- "testIndLogistic"   
     }
     
-    ## ordinal, multinomial or perhaps binary data
-    if ( is.factor(target) ) {
-      if ( !is.ordered(target) ) {
-        if ( length(unique(target) ) == 2 ) {
-          target <- as.vector(target)
-          test <- "binary"
-        } else {
-          test <- "multinomial"
-        }  
-
-      } else {
-        if ( length(unique(target) ) == 2 ) {
-          target <- as.vector(target)
-          test <- "binary"
-        } else {
-          test <- "ordinal"    
-        }
-      }
-    }
-    
-    if ( is.vector(target) ) {
-      
-      if ( length( unique(target) ) > 2  &  sum( round(target) - target ) == 0 ) {
-        test <- "poisson"
-        
-        ## binomial regression 
-        
-      } else if ( length( unique(target) ) == 2 ) {
-        test <- "binary"
-        
-        ## linear regression 
-      } else if ( sum( class(target) == "numeric" ) > 0  || sum( class(target) == "vector" ) > 0 ) {
-        test <- "gaussian"  
-      }
-    } 
-  
+ 
   }
 
 
     #available conditional independence tests
-    av_models = c("gaussian", "median", "beta", "Cox", "Weibull", "binary", "multinomial", "ordinal", "poisson", "nb", "zip", "speedglm");
+    av_models = c("testIndReg", "testIndRQ", "testIndBeta", "testIndCR", "testIndWR", "testIndLogistic", "testIndPois", "testIndNB", "testIndZIP", "testIndSpeedglm");
     
-    ci_model = test
     #cat(test)
     
-  if ( test == "binary" || test == "poisson" ||  ( test == "gaussian"  &  !is.matrix(target) ) || ( test == "gaussian" ) ) {
+  if ( ( test == "testIndLogistic"  &  length( Rfast::sort_unique(target) ) == 2 )  ||  test == "testIndPois"  ||  ( test == "testIndReg"  &  !is.matrix(target) ) ) {
    
-    result <- bic.glm.fsreg( target, dataset, test = test, robust = robust, tol = tol, ncores = ncores ) 
-
+    result <- bic.glm.fsreg( target, dataset, wei = wei, tol = tol, heavy = FALSE, robust = robust, ncores = ncores ) 
+  
+  } else if ( test == "testIndspeedglm" ) {
+    
+    result <- bic.glm.fsreg( target, dataset, wei = wei, tol = tol, heavy = TRUE, robust = robust, ncores = ncores ) 
+    
   } else {
  
-    test = match.arg(test , av_models ,TRUE);
+    ci_test <- test <- match.arg(test, av_models ,TRUE);
     #convert to closure type
     
-    if ( test == "beta" ) {
-      test = betareg 
+    if ( test == "testIndBeta" ) {
+      test = betareg::betareg 
       robust = FALSE
 
-    } else if ( test == "Cox" ) {
-      test = coxph 
+    } else if ( test == "censIndCR") {
+      test = survival::coxph 
       robust = FALSE
 
-    } else if ( test == "multinomial" ) {
-      test = multinom 
+	} else if ( test == "censIndWR" ) {
+      test = survival::survreg 
+      robust = FALSE
+	  
+    } else if ( test == "testIndLogistic" ) {
+	  if ( is.ordered(target) ) {      
+        test = ordinal::clm
+        robust = FALSE
+      } else {
+		    test = nnet::multinom  
+	      robust = FALSE
+	  }
+
+    } else if ( test == "testIndNB" ) {
+      test = MASS::glm.nb
       robust = FALSE
 
-    } else if ( test == "ordinal" ) {
-      test = clm 
+    } else if ( test == "testIndZIP" ) {
+      test = pscl::zeroinfl 
+      robust = FALSE
+	  
+	} else if ( test == "testIndRQ" ) {
+      test = quantreg::rq 
       robust = FALSE
 
-    } else if ( test == "neg.bin" ) {
-      test = glm.nb
-      robust = FALSE
-
-    } else if ( test == "zip" ) {
-      test = zeroinfl 
-      robust = FALSE
-
-    } else if ( test == "gaussian"  &  is.matrix(target) ) {
-      test = lm 
-      robust = FALSE
+    } else if ( test == "testIndReg"  &  !is.matrix(target) ) {
+	    if (robust == FALSE) {
+        test = lm 
+      } else  test = MASS::rlm
+	
     }
 
     
-    
     runtime <- proc.time()
 
       ini = test( target ~ 1 )
@@ -174,8 +154,9 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
     
     if (ncores <= 1) {
         for (i in 1:p) {
-          mi <- test( target ~ dataset[, i] )
-          bico[i] <-  - 2 * as.numeric( logLik(mi) ) + length( coef( mi ) ) * con
+          mi <- test( target ~ dataset[, i], weights = wei )
+		      la <- logLik(mi)
+          bico[i] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
         }
 
       mat <- cbind(1:p, bico)
@@ -189,8 +170,9 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
       registerDoParallel(cl)
       bico <- numeric(p)
       mod <- foreach( i = 1:p, .combine = rbind) %dopar% {
-        ww <- test( target ~ dataset[, i] )
-        bico[i] <-  - 2 * as.numeric( logLik(ww) ) + length( coef( ww ) ) * con
+        ww <- test( target ~ dataset[, i], weights = wei )
+		la <- logLik(ww)
+        bico[i] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
       }
       stopCluster(cl)
 
@@ -205,24 +187,23 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
     colnames(mat) <- c("variable", "BIC")
     rownames(mat) <- 1:p
     sel <- which.min( mat[, 2] )
-    sela <- sel
-
+    
+    
     if ( mat[sel, 2] < ini ) {
 
       info[1, ] <- mat[sel, ]
       mat <- mat[-sel, ]
-      if ( !is.matrix(mat) ) {
-        mat <- matrix(mat, ncol = 2) 
-      }
+      if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 2) 
       mat <- mat[ order( mat[, 2] ), ]
-     
-      mi = test( target ~ dataset[, sel] )
-      tool[1] <-  - 2 * as.numeric( logLik(mi) ) + length( coef( mi ) ) * con
-      if ( is.na(tool[1]) )  tool[1] <- ini
+      sela <- sel
+      
+      mi <- test( target ~ dataset[, sel], weights = wei )
+      la <- logLik(mi)
+      tool[1] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
 
       moda[[ 1 ]] <- mi
 
-    }
+    } 
 
 
     ######
@@ -239,8 +220,9 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
       if ( ncores <= 1 ) {
         bico <- numeric( pn )
         for ( i in 1:pn ) {
-          ma <- test( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ) )
-          bico[i] <-  - 2 * as.numeric( logLik(ma) ) + length( coef( ma ) ) * con
+          ma <- test( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
+		  la <- logLik(ma)
+          bico[i] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
         }
 
         mat[, 2] <- bico
@@ -251,8 +233,9 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
         registerDoParallel(cl)
         bico <- numeric(pn)
         mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-          ww <- test( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ) )
-          bico[i] <-  - 2 * as.numeric( logLik(ww) ) + length( coef( ww ) ) * con
+          ww <- test( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
+          la <- logLik(ww)
+          bico[i] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
         }
 
         stopCluster(cl)
@@ -265,28 +248,23 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
       sel <- mat[ina, 1]
 
       if ( mat[ina, 2] >= tool[1] ) {
-        info <- rbind( info,  c( -10, 1e300 ) )
+        info <- info
 
       } else {
         tool[2] <- mat[ina, 2]
         info <- rbind(info, mat[ina, ] )
         sela <- info[, 1]
         mat <- mat[-ina , ]
-        if ( !is.matrix(mat) ) {
-          mat <- matrix(mat, ncol = 2) 
-        }
+        if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 2) 
         mat <- mat[ order( mat[, 2] ), ]
 
-        mi = test( target ~., data = as.data.frame( dataset[, sela] ) )
-        tool[2] <-  - 2 * as.numeric( logLik(mi) ) + length( coef( mi ) ) * con
+        mi <- test( target ~., data = as.data.frame( dataset[, sela] ), weights = wei )
+	      la <- logLik(mi)
+        tool[2] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
         moda[[ 2 ]] <- mi
 
       }
 
-      if ( sum( info[2, ] -  c( -10, 1e300 ) ) == 0 ) {
-        info <- matrix( info[1, ], ncol = 2)
-      }
-
    }
 
 
@@ -303,8 +281,9 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
 
         if (ncores <= 1) {
           for ( i in 1:pn ) {
-            ma <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ) )
-            mat[i, 2] <-  - 2 * as.numeric( logLik(ma) ) + length( coef( ma ) ) * con
+            ma <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
+		    la <- logLik(ma)
+            mat[i, 2] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
           }
 
         } else {
@@ -312,8 +291,9 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
           registerDoParallel(cl)
           bico <- numeric(pn)
           mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-            ww <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ) )
-            bico[i] <- - 2 * as.numeric( logLik(ww) ) + length( coef( ww ) ) * con
+            ww <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
+			la <- logLik(ww)
+            bico[i] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
           }
 
           stopCluster(cl)
@@ -331,17 +311,16 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
 
         } else {
 
-          tool[k] = mat[ina, 2]
+          tool[k] <- mat[ina, 2]
           info <- rbind(info, mat[ina, ] )
           sela <- info[, 1]
           mat <- mat[-ina , ]
-          if ( !is.matrix(mat) ) {
-            mat <- matrix(mat, ncol = 2) 
-          }
+          if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 2) 
           mat <- mat[ order( mat[, 2] ), ]
 
-          ma = test( target ~., data =as.data.frame( dataset[, sela] ) )
-          tool[k] <-  - 2 * as.numeric( logLik(ma) ) + length( coef( ma ) ) * con
+          ma <- test( target ~., data =as.data.frame( dataset[, sela] ), weights = wei )
+		      la <- logLik(ma)
+          tool[k] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
 
           moda[[ k ]] <- ma
 
@@ -367,11 +346,11 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
         xx <- as.data.frame( dataset[, sela] )
         colnames(xx) <- paste("V", sela, sep = "")
 
-        models <- final <- test( target ~., data = as.data.frame( xx ) )
+        models <- final <- test( target ~., data = as.data.frame( xx ), weights = wei )
 
       } else {
         for (i in 1:d) {
-          models[[ i ]] <- test( target ~., data = as.data.frame( xx[, 1:i] ) )
+          models[[ i ]] <- test( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
         }
 
         final <- summary( models[[ d ]] )
@@ -385,7 +364,7 @@ bic.fsreg <- function( target, dataset, test = NULL, robust = FALSE, tol = 0, nc
 
     }
 
-    result = list(mat = t(mat), info = info, models = models, final = final, runtime = runtime )
+    result = list(mat = t(mat), info = info, models = models, final = final, ci_test = ci_test, runtime = runtime )
 
   } 
 
