@@ -1,4 +1,4 @@
-censIndCR = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateModels=NULL, hash = FALSE, stat_hash=NULL, pvalue_hash=NULL,robust=FALSE){
+censIndCR = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL, univariateModels=NULL, hash = FALSE, stat_hash=NULL, pvalue_hash=NULL,robust=FALSE){
   # Conditional independence test based on the Log Likelihood ratio test
   
   if(!survival::is.Surv(target))
@@ -63,7 +63,7 @@ censIndCR = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariate
     
     #fitting the model
     tryCatch(
-      cox_results <- survival::coxph(target ~ x),
+      cox_results <- survival::coxph(target ~ x, weights = wei),
       warning=function(w) {
         #Do nothing...
       }
@@ -73,12 +73,7 @@ censIndCR = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariate
     }
     
     #retrieve the p value and stat. 
-    if ( is.factor(x) ) {
-      dof = nlevels(x) - 1   
-    } else{
-      dof = 1
-    }
-    
+    dof <- length( coef(cox_results) )
     stat = cox_results$score;
     pvalue = pchisq(stat, dof, lower.tail = FALSE, log.p = TRUE);
     
@@ -118,7 +113,7 @@ censIndCR = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariate
     tryCatch(
       
       #fitting the full model
-      cox_results_full <- survival::coxph(target ~ ., data = as.data.frame(  dataset[ , c(csIndex, xIndex)] ) ),
+      cox_results_full <- survival::coxph(target ~ ., data = as.data.frame(  dataset[ , c(csIndex, xIndex)] ), weights = wei ),
       
       warning=function(w) {
         #Do nothing
