@@ -1,4 +1,4 @@
-testIndClogit = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univariateModels=NULL, hash = FALSE, stat_hash=NULL, pvalue_hash=NULL,robust=FALSE){
+testIndClogit = function(target, dataset, xIndex, csIndex, wei =  NULL, dataInfo=NULL, univariateModels=NULL, hash = FALSE, stat_hash=NULL, pvalue_hash=NULL,robust=FALSE){
   # Conditional independence test based on the Log Likelihood ratio test
   
   if( class(target)!= "matrix" || ( class(target)== "matrix" & ncol(target)!=2 ) )
@@ -60,7 +60,7 @@ testIndClogit = function(target, dataset, xIndex, csIndex, dataInfo=NULL, univar
     
     #fitting the model
     tryCatch(
-      clogit_results <- survival::clogit(case ~ x + strata(id)),
+      clogit_results <- survival::clogit(case ~ x + strata(id) ,weights = wei),
       warning=function(w) {
         #Do nothing...
       }
